
# Create a model
resource "aws_sagemaker_model" "model" {
  count = var.skip_model_creation ? 0 : 1

  name = var.model_name

  execution_role_arn = var.execution_role_arn

  primary_container {
    image          = var.container_image
    model_data_url = length(var.model_bucket_name) > 0 ? format("s3://%s/%s", var.model_bucket_name, var.model_bucket_path) : null
    environment    = var.model_env_vars
  }

  tags = var.tags
}

# Create endpoint config
resource "aws_sagemaker_endpoint_configuration" "ec" {
  name = var.endpoint_config_name

  production_variants {
    variant_name           = length(var.production_variant_name) > 0 ? var.production_variant_name : null
    model_name             = var.skip_model_creation ? var.model_name : aws_sagemaker_model.model[0].name
    initial_instance_count = var.initial_instance_count
    instance_type          = var.instance_type
  }

  tags = var.tags
}

# Create endpoint
resource "aws_sagemaker_endpoint" "ep" {
  name = var.endpoint_name

  endpoint_config_name = aws_sagemaker_endpoint_configuration.ec.name

  tags = var.tags
}

resource "aws_appautoscaling_target" "sagemaker_target" {
  min_capacity       = var.min_capacity
  max_capacity       = var.max_capacity
  resource_id        = "endpoint/${aws_sagemaker_endpoint.ep.name}/variant/${aws_sagemaker_endpoint_configuration.ec.production_variants[0].variant_name}"
  role_arn           = var.execution_role_arn
  scalable_dimension = "sagemaker:variant:DesiredInstanceCount"
  service_namespace  = "sagemaker"
}

resource "aws_appautoscaling_policy" "sagemaker_policy" {
  name               = "${aws_sagemaker_endpoint.ep.name}-scaling-policy"
  policy_type        = var.policy_type
  resource_id        = aws_appautoscaling_target.sagemaker_target.resource_id
  scalable_dimension = aws_appautoscaling_target.sagemaker_target.scalable_dimension
  service_namespace  = aws_appautoscaling_target.sagemaker_target.service_namespace


  target_tracking_scaling_policy_configuration {
    predefined_metric_specification {
      predefined_metric_type = "SageMakerVariantInvocationsPerInstance"
    }
    target_value       = var.target_value
    scale_in_cooldown  = var.scale_in_cooldown
    scale_out_cooldown = var.scale_out_cooldown
    disable_scale_in   = var.disable_scale_in
  }
}